######################################
# File with Functions and Structures #
######################################

# Structure for parameters
mutable struct ParamsStruc{T <: Real}
    # Demand curve
    α::T        # constant
    β::T        # slope
    # Cost function
    γ::T        # quadratic term
    # Grid of output
    ogrid::AbstractArray
end

################################################################################

# Set parameter values
function SetParams(; α = 1.0, β = 0.5, γ = 0.5)
    # Construct grid of output points
    ogrid = range(0.0,step=α/(β*100.0),stop=α/β)
    # Output
    ParamsStruc(α,β,γ,ogrid)
end

################################################################################

# Negative of the monopolist's profits
# This specifies the objective function used in the maximization routines
function NProfits(output,Params)
    # Unpack parameters
    @unpack α,β,γ = Params
    # Firm's revenue
    rev   = α.*output .- β.*(output.^2.0)./2.0
    # Firm's costs
    costs = γ.*(output.^2.0)/2.0
    # Firm's profits
    π = rev .- costs
    # Switch sign for minimizing
    return .-π
end

# Function to be solved for
NegProfits(x) = NProfits(x,Params)

################################################################################
