#########################
# Main file to run code #
#########################

# Housekeeping
exit()
clearconsole()

# Load packages
using LinearAlgebra, Optim, Parameters, Plots

# Load functions
include("Functions/f_Functions.jl")

# Call exogenous parameters
Params = SetParams()

# Plot marginal revenue and marginal cost
plot(Params.ogrid, .-NProfits(Params.ogrid,Params),
    title = "Objective Function", titlefontsize=16,
    ylabel="Profits", ytickfontsize=12, yguidefontsize=12,
    xlabel="Output", xtickfontsize=12,
    lw=3, legend=false)

## Call up minimizer to solve for output

# Lower bound
x_lb = 0.0
# Upper bound
x_ub = Params.α/Params.β
# Call solver
results = Optim.optimize(NegProfits,x_lb,x_ub)
# Optimal output
output = results.minimizer

# Compute additional variables
price = Params.α - Params.β*output/2.0
cost = Params.γ*(output^2.0)/2.0
profits = price*output - cost
markup = price/(Params.γ*output)

# Display results
print("Results for the monopoly model \n")
println("Output = $output, Price = $price, "*
        "Profits = $profits, and Markups = $markup \n")

## Discrete maximization

# Discretize domain for maximization
oset = x_lb:2.0/99.0:x_ub
# Maximal value of objective function
maxvalue = maximum(.-NegProfits(oset))
# Point number of the optimal value of output
opoint = argmax(.-NegProfits(oset))
# Optimal level of output
output = oset[opoint]

# Compute additional variables
price = Params.α - Params.β*output/2.0
cost = Params.γ*(output^2.0)/2.0
profits = price*output - cost
markup = price/(Params.γ*output)

# Display results
print("Results for the monopoly model: Discrete maximization \n")
println("Output = $output, Price = $price, "*
        "Profits = $profits, and Markups = $markup \n")


## Call up particle swarm algorithm to solve for output
# Only available for multivariate functions

# # Guess
# x_guess = 0.2
# # Call solver
# results = Optim.optimize(NegProfits, x_guess, ParticleSwarm(; lower = x_lb,
#                         upper = x_ub, n_particles = 1))
# # Optimal output
# output = results.minimizer
#
# # Compute additional variables
# price = Params.α - Params.β*output/2.0
# cost = Params.γ*(output^2.0)/2.0
# profits = price*output - cost
# markup = price/(Params.γ*output)
#
# # Display results
# print("Results for the monopoly model: Particle swarm maximization \n")
# println("Output = $output, Price = $price, "*
#         "Profits = $profits, and Markups = $markup \n")
